package pwc.taxtech.atms.service.impl;

import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.analysis.dao.*;
import pwc.taxtech.atms.analysis.entity.*;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.common.message.ErrorMessageCN;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.EnumAnalysisExpTempPath;
import pwc.taxtech.atms.constant.enums.EnumAnalysisImportType;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.analysis.*;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.OrganizationExample;
import pwc.taxtech.atms.exception.ServiceException;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Auther: Gary J Li
 * @Date: 14/03/2019 11:31
 * @Description:
 */
@Service
public class AnalysisServiceImpl extends BaseService {

    @Resource
    private AnalysisDriverNumMapper analysisDriverNumMapper;
    @Resource
    private AnalysisEmployeeNumMapper analysisEmployeeNumMapper;
    @Resource
    private AnalysisExpectedTaxReturnMapper analysisExpectedTaxReturnMapper;
    @Resource
    private AnalysisActualTaxReturnMapper analysisActualTaxReturnMapper;
    @Resource
    private AnalysisFeeMapper analysisFeeMapper;
    @Resource
    private AnalysisFileManagementMapper analysisFileManagementMapper;
    @Resource
    private AnalysisGmvSubsidyMapper analysisGmvSubsidyMapper;
    @Resource
    private AnalysisInternationalBusinessDataMapper analysisInternationalBusinessDataMapper;
    @Resource
    private AnalysisInternationalTaxDataMapper analysisInternationalTaxDataMapper;
    @Resource
    private AnalysisMasterMapper analysisMasterMapper;
    @Resource
    private AnalysisSalesMapper analysisSalesMapper;
    @Resource
    private AnalysisTaxMapper analysisTaxMapper;
    @Resource
    private OrganizationMapper organizationMapper;

    public List<Object> displayAnalysisImportData(AnalysisDomesticlParam param) {
        List<Object> objects = new ArrayList<>();
        Integer period = DateUtils.strToPeriod(param.getPeriod());
        switch (param.getType()){
            case 0:
                AnalysisTaxExample analysisTaxExample = new AnalysisTaxExample();
                analysisTaxExample.createCriteria().andPeriodEqualTo(period);
                List<AnalysisTax> list = analysisTaxMapper.selectByExample(analysisTaxExample);
                List<AnalysisTaxDto> dtos = Lists.newArrayList();
                Map<String, List<AnalysisTax>> actrs =
                        list.stream().collect(Collectors.groupingBy(AnalysisTax::getCompanyName));
                actrs.forEach((k, v) -> processAts(k, v,dtos));
                objects.addAll(dtos);
                break;
            case 1:
                AnalysisActualTaxReturnExample analysisActualTaxReturnExample = new AnalysisActualTaxReturnExample();
                analysisActualTaxReturnExample.createCriteria().andPeriodEqualTo(period);
                List<AnalysisActualTaxReturn> list2 = analysisActualTaxReturnMapper.selectByExample(analysisActualTaxReturnExample);
                List<AnalysisActualTaxReturnDto> dtos2 = Lists.newArrayList();
                Map<String, List<AnalysisActualTaxReturn>> actrs2 =
                        list2.stream().collect(Collectors.groupingBy(AnalysisActualTaxReturn::getCompanyName));
                actrs2.forEach((k, v) -> processActrs(k, v,dtos2));
                objects.addAll(dtos2);
                return objects;
            case 2:
                AnalysisGmvSubsidyExample analysisGmvSubsidyExample = new AnalysisGmvSubsidyExample();
                analysisGmvSubsidyExample.createCriteria().andPeriodEqualTo(period);
                List<AnalysisGMVSubsidyDto> dtos3 = generalDtos(analysisGmvSubsidyMapper.selectByExample(analysisGmvSubsidyExample));
                objects.addAll(dtos3);
                break;
            case 3:
                AnalysisEmployeeNumExample analysisEmployeeNumExample = new AnalysisEmployeeNumExample();
                analysisEmployeeNumExample.createCriteria().andPeriodEqualTo(period);
                objects.addAll(analysisEmployeeNumMapper.selectByExample(analysisEmployeeNumExample));
                break;
            case 4:
                AnalysisDriverNumExample analysisDriverNumExample = new AnalysisDriverNumExample();
                analysisDriverNumExample.createCriteria().andPeriodEqualTo(period);
                List<AnalysisDriverNumDto> dtos4 = processDns(analysisDriverNumMapper.selectByExample(analysisDriverNumExample));
                objects.addAll(dtos4);
                break;
            default:
                break;
        }
        return objects;
    }

    private List<AnalysisGMVSubsidyDto> generalDtos(List<AnalysisGmvSubsidy> list) {
        List<AnalysisGMVSubsidyDto> dtos = Lists.newArrayList();
        NumberFormat nt = NumberFormat.getPercentInstance();
        list.forEach(l -> {
            AnalysisGMVSubsidyDto dto = new AnalysisGMVSubsidyDto();
            dto.setBusinessLine(l.getBusinessLine());
            dto.setSegment1(nt.format(l.getOrderChainRatio().doubleValue()));
            dto.setSegment2(nt.format(l.getGmvChainRatio().doubleValue()));
            dto.setSegment3(nt.format(l.getbSubsidyRate().doubleValue()));
            dto.setSegment4(nt.format(l.getbEndLinkRatio().doubleValue()));
            dto.setSegment5(nt.format(l.getcSubsidyRate().doubleValue()));
            dto.setSegment6(nt.format(l.getcEndLinkRatio().doubleValue()));
            dtos.add(dto);
        });
        return dtos;
    }

    private void processAts(String k, List<AnalysisTax> v, List<AnalysisTaxDto> dtos) {
        AnalysisTaxDto dto = new AnalysisTaxDto();
        dto.setCompanyName(k);
        dto.setSegment1(getAmountByGroup(v,"城建税"));
        dto.setSegment2(getAmountByGroup(v,"教育费附加"));
        dto.setSegment3(getAmountByGroup(v,"地方教育费附加"));
        dto.setSegment4(getAmountByGroup(v,"员工个税"));
        dto.setSegment5(getAmountByGroup(v,"司机个税"));
        dto.setSegment6(getAmountByGroup(v,"印花税"));
        dtos.add(dto);
    }

    private void processActrs(String k, List<AnalysisActualTaxReturn> v,List<AnalysisActualTaxReturnDto> dtos) {
        AnalysisActualTaxReturnDto dto = new AnalysisActualTaxReturnDto();
        dto.setCompanyName(k);
        dto.setSegment1(getTaxAmountByGroup(v,"增值税返还"));
        dto.setSegment2(getTaxAmountByGroup(v,"城建税返还"));
        dto.setSegment3(getTaxAmountByGroup(v,"教育费附加返还"));
        dto.setSegment4(getTaxAmountByGroup(v,"地方教育费附加返还"));
        dto.setSegment5(getTaxAmountByGroup(v,"个人所得税返还"));
        dtos.add(dto);
    }

    private List<AnalysisDriverNumDto> processDns(List<AnalysisDriverNum> list) {
        List<AnalysisDriverNumDto> dtos = Lists.newArrayList();
        list.forEach(l->{
            AnalysisDriverNumDto dto1 = new AnalysisDriverNumDto();
            dto1.setDriverType("加盟");
            dto1.setDriverNum(l.getJoinInAmount());
            AnalysisDriverNumDto dto2 = new AnalysisDriverNumDto();
            dto2.setDriverType("自营");
            dto2.setDriverNum(l.getSelfSupportAmount());
            AnalysisDriverNumDto dto3 = new AnalysisDriverNumDto();
            dto3.setDriverType("直营");
            dto3.setDriverNum(l.getDirectSaleAmount());
            AnalysisDriverNumDto dto4 = new AnalysisDriverNumDto();
            dto4.setDriverType("对公");
            dto4.setDriverNum(l.getRightPublicAmount());
            dtos.add(dto1);
            dtos.add(dto2);
            dtos.add(dto3);
            dtos.add(dto4);
        });
        return dtos;
    }

    private BigDecimal getTaxAmountByGroup(List<AnalysisActualTaxReturn> v, String group) {
        List<AnalysisActualTaxReturn> filterRes = v.stream().filter(o-> group.equals(o.getTaxReturnGroup())).collect(Collectors.toList());
        BigDecimal res = filterRes.size()>0?filterRes.get(0).getTaxReturnAmount():BigDecimal.ZERO;
        return res;
    }

    private BigDecimal getAmountByGroup(List<AnalysisTax> v, String group) {
        List<AnalysisTax> filterRes = v.stream().filter(o-> group.equals(o.getTaxGroup())).collect(Collectors.toList());
        BigDecimal res = filterRes.size()>0?filterRes.get(0).getTaxAmount():BigDecimal.ZERO;
        return res;
    }

    public OperationResultDto importDomesitcExcelFile(MultipartFile file, String periodDate, Integer type) {
        switch (type){
            case 0:
                importAnalysisTaxExcelFile(file,periodDate);
                break;
            case 1:
                importAnalysisReturnTaxExcelFile(file,periodDate);
                break;
            case 2:
                importAnalysisGMVSubsidyExcelFile(file,periodDate);
                break;
            case 3:
                importAnalysisEmployeeNumExcelFile(file,periodDate);
                break;
            case 4:
                importAnalysisDriverNumExcelFile(file,periodDate);
                break;
            default:
                break;
        }
        return OperationResultDto.success();
    }

    public OperationResultDto importInterNationalExcelFile(MultipartFile file, String periodDate, Integer type,
                                                           String companyName,String country) {
        switch (type){
            case 100:
                importAnalysisInterBuDataExcelFile(file,periodDate,companyName,country);
                break;
            case 101:
                importAnalysisInterTaxDataExcelFile(file,periodDate,companyName,country);
                break;
            default:
                break;
        }
        return OperationResultDto.success();
    }

    private void importAnalysisTaxExcelFile(MultipartFile file, String periodDate) {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            // 文件上的期间
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisTax> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    String companyName = getCellStringValue(cell1);
                    OrganizationExample example = new OrganizationExample();
                    example.createCriteria().andNameEqualTo(companyName);
                    List<Organization> orgs = organizationMapper.selectByExample(example);
                    if(orgs.isEmpty()){
                        break;
                    }
                    for (int k = 1; k < sheet.getRow(0).getLastCellNum(); k++) {
                        AnalysisTax model = getAnalysisTax(selectedPer, sheet, k, j, orgs.get(0));
                        lists.add(model);
                    }
                }
                AnalysisTaxExample example = new AnalysisTaxExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisTaxMapper.deleteByExample(example);
                lists.forEach(l -> {
                    analysisTaxMapper.insertSelective(l);
                });
            }
        } catch (ServiceException se) {
            throw se;
        } catch (Exception e) {
            String errMsg = "分析模块-导入各税种税额异常";
            logger.error(errMsg, e);
            throw new ServiceException(errMsg);
        }
    }

    private AnalysisTax getAnalysisTax(Integer period, Sheet sheet,int k, int j,Organization org) {
        AnalysisTax model = new AnalysisTax();
        model.setId(idService.nextId());
        model.setOrganizationId(org.getId());
        model.setSeqNo(org.getCode()+""+period);
        model.setPeriod(period);
        model.setCompanyName(org.getName());
        model.setTaxGroup(getCellStringValue(sheet.getRow(0).getCell(k)));
        model.setTaxAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(k)));
        return model;
    }

    private void importAnalysisReturnTaxExcelFile(MultipartFile file, String periodDate) {
        try{
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisActualTaxReturn> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    String companyName = getCellStringValue(cell1);
                    OrganizationExample example = new OrganizationExample();
                    example.createCriteria().andNameEqualTo(companyName);
                    List<Organization> orgs = organizationMapper.selectByExample(example);
                    if(orgs.isEmpty()){
                        break;
                    }
                    for(int k = 1; k < sheet.getRow(0).getLastCellNum(); k++){
                        AnalysisActualTaxReturn model = getAnalysisActualTaxReturn(selectedPer, sheet,k, j, orgs.get(0));
                        lists.add(model);
                    }
                }
                AnalysisActualTaxReturnExample example = new AnalysisActualTaxReturnExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisActualTaxReturnMapper.deleteByExample(example);
                lists.forEach(l->{
                    analysisActualTaxReturnMapper.insertSelective(l);
                });
            }
        }catch (ServiceException se){
            throw se;
        }catch (Exception e){
            String errMsg = "分析模块-导入实际返还税额异常";
            logger.error(errMsg,e);
            throw new ServiceException(errMsg);
        }
    }

    private AnalysisActualTaxReturn getAnalysisActualTaxReturn(Integer period, Sheet sheet,int k, int j, Organization org) {
        AnalysisActualTaxReturn model = new AnalysisActualTaxReturn();
        model.setId(idService.nextId());
        model.setOrganizationId(org.getId());
        model.setSeqNo(org.getCode()+""+period);
        model.setPeriod(period);
        model.setCompanyName(org.getName());
        model.setTaxReturnGroup(getCellStringValue(sheet.getRow(0).getCell(k)));
        model.setTaxReturnAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(k)));
        return model;
    }

    private void importAnalysisEmployeeNumExcelFile(MultipartFile file, String periodDate) {
        try{
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisEmployeeNum> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {

                    AnalysisEmployeeNum model = new AnalysisEmployeeNum();
                    model.setId(idService.nextId());
                    model.setPeriod(selectedPer);

                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    model.setFullTimeAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(1)));
                    model.setInternAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(2)));
                    model.setVendorAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(3)));
                    model.setTotalAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(4)));
                    lists.add(model);
                }
                AnalysisEmployeeNumExample example = new AnalysisEmployeeNumExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisEmployeeNumMapper.deleteByExample(example);
                lists.forEach(l->{
                    analysisEmployeeNumMapper.insertSelective(l);
                });
            }
        }catch (ServiceException se){
            throw se;
        }catch (Exception e){
            String errMsg = "分析模块-导入雇员人数异常";
            logger.error(errMsg,e);
            throw new ServiceException(errMsg);
        }
    }

    private void importAnalysisGMVSubsidyExcelFile(MultipartFile file, String periodDate) {
        try{
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisGmvSubsidy> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    AnalysisGmvSubsidy model = new AnalysisGmvSubsidy();
                    model.setId(idService.nextId());
                    model.setPeriod(selectedPer);
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    model.setBusinessLine(getCellStringValue(cell1));
                    model.setOrderChainRatio(getCellBigDecimalValue(sheet.getRow(j).getCell(1)));
                    model.setGmvChainRatio(getCellBigDecimalValue(sheet.getRow(j).getCell(2)));
                    model.setbSubsidyRate(getCellBigDecimalValue(sheet.getRow(j).getCell(3)));
                    model.setbEndLinkRatio(getCellBigDecimalValue(sheet.getRow(j).getCell(4)));
                    model.setcSubsidyRate(getCellBigDecimalValue(sheet.getRow(j).getCell(5)));
                    model.setcEndLinkRatio(getCellBigDecimalValue(sheet.getRow(j).getCell(6)));
                    lists.add(model);
                }
                AnalysisGmvSubsidyExample example = new AnalysisGmvSubsidyExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisGmvSubsidyMapper.deleteByExample(example);
                lists.forEach(l->{
                    analysisGmvSubsidyMapper.insertSelective(l);
                });
            }
        }catch (ServiceException se){
            throw se;
        }catch (Exception e){
            String errMsg = "分析模块-导入GMV及补贴异常";
            logger.error(errMsg,e);
            throw new ServiceException(errMsg);
        }
    }

    private void importAnalysisDriverNumExcelFile(MultipartFile file, String periodDate) {
        try{
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                AnalysisDriverNum model = new AnalysisDriverNum();
                model.setId(idService.nextId());
                model.setPeriod(selectedPer);
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    Cell cell2 = sheet.getRow(j).getCell(1);
                    if("加盟".equals(getCellStringValue(cell1))){
                        model.setJoinInAmount(getCellBigDecimalValue(cell2));
                    }else if("自营".equals(getCellStringValue(cell1))){
                        model.setSelfSupportAmount(getCellBigDecimalValue(cell2));
                    }else if("直营".equals(getCellStringValue(cell1))){
                        model.setDirectSaleAmount(getCellBigDecimalValue(cell2));
                    }else if("对公".equals(getCellStringValue(cell1))){
                        model.setRightPublicAmount(getCellBigDecimalValue(cell2));
                    }
                }
                AnalysisDriverNumExample example = new AnalysisDriverNumExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisDriverNumMapper.deleteByExample(example);
                analysisDriverNumMapper.insertSelective(model);
            }
        }catch (ServiceException se){
            throw se;
        }catch (Exception e){
            String errMsg = "分析模块-导入司机人数异常";
            logger.error(errMsg,e);
            throw new ServiceException(errMsg);
        }
    }

    private void importAnalysisInterTaxDataExcelFile(MultipartFile file, String periodDate,
                                                     String companyName,String country) {
        try{
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            // 文件上的期间
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisInternationalTaxData> lists = Lists.newArrayList();
                for (int j = 1; j <= 1; j++) {
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    AnalysisInternationalTaxData model = new AnalysisInternationalTaxData();
                    model.setId(idService.nextId());
                    model.setPeriod(selectedPer);
                    model.setCompanyName(companyName);
                    model.setCountry(country);
                    model.setTaxCategory(getCellStringValue(sheet.getRow(j).getCell(0)));
                    model.setTaxType(getCellStringValue(sheet.getRow(j).getCell(1)));
                    model.setTaxAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(2)));
                    lists.add(model);
                }
                AnalysisInternationalTaxDataExample example = new AnalysisInternationalTaxDataExample();
                example.createCriteria().andPeriodEqualTo(selectedPer).andCountryEqualTo(country).andCompanyNameEqualTo(companyName);
                analysisInternationalTaxDataMapper.deleteByExample(example);
                lists.forEach(l->{
                    analysisInternationalTaxDataMapper.insertSelective(l);
                });
            }
        }catch (ServiceException se){
            throw se;
        }catch (Exception e){
            String errMsg = "分析模块-导入国际税务数据异常";
            logger.error(errMsg,e);
            throw new ServiceException(errMsg);
        }
    }

    private void importAnalysisInterBuDataExcelFile(MultipartFile file, String periodDate,
                                                    String companyName,String country) {
        try{
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            // 文件上的期间
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisInternationalBusinessData> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    AnalysisInternationalBusinessData model = new AnalysisInternationalBusinessData();
                    model.setId(idService.nextId());
                    model.setPeriod(selectedPer);
                    model.setCompanyName(companyName);
                    model.setCountry(country);
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    model.setGmv(getCellBigDecimalValue(cell1));
                    model.setTrips(getCellBigDecimalValue(sheet.getRow(j).getCell(1)));
                    model.setSubsidyB(getCellBigDecimalValue(sheet.getRow(j).getCell(2)));
                    model.setSubsidyC(getCellBigDecimalValue(sheet.getRow(j).getCell(3)));
                    model.setRevenue(getCellBigDecimalValue(sheet.getRow(j).getCell(4)));
                    model.setProfit(getCellBigDecimalValue(sheet.getRow(j).getCell(5)));
                    lists.add(model);
                }
                AnalysisInternationalBusinessDataExample example = new AnalysisInternationalBusinessDataExample();
                example.createCriteria().andPeriodEqualTo(selectedPer).andCountryEqualTo(country).andCompanyNameEqualTo(companyName);
                analysisInternationalBusinessDataMapper.deleteByExample(example);
                lists.forEach(l->{
                    analysisInternationalBusinessDataMapper.insertSelective(l);
                });
            }
        }catch (ServiceException se){
            throw se;
        }catch (Exception e){
            String errMsg = "分析模块-导入国际业务数据异常";
            logger.error(errMsg,e);
            throw new ServiceException(errMsg);
        }
    }

    public List<Object> displayAnalysisInternationalImportData(AnalysisInternationlParam param) {
        List<Object> objects = new ArrayList<>();
        Integer period = DateUtils.strToPeriod(param.getPeriod());
        String country = param.getCountry();
        String companyName = param.getCompanyName();
        switch (param.getType()){
            case 100:
                AnalysisInternationalBusinessDataExample analysisInternationalBusinessDataExample = new AnalysisInternationalBusinessDataExample();
                analysisInternationalBusinessDataExample.createCriteria().andPeriodEqualTo(period).andCompanyNameEqualTo(companyName).andCountryEqualTo(country);
                List<AnalysisInternationalBusinessData> list = analysisInternationalBusinessDataMapper.selectByExample(analysisInternationalBusinessDataExample);

                /*List<AnalysisInternationalBUDataDto> dtos = Lists.newArrayList();
                list.forEach(l->{
                    AnalysisInternationalBUDataDto dto1 = new AnalysisInternationalBUDataDto();
                    dto1.setCategory("GMV");
                    dto1.setName("GMV");
                    dto1.setAmount(l.getGmv());
                    dtos.add(dto1);
                    AnalysisInternationalBUDataDto dto2 = new AnalysisInternationalBUDataDto();
                    dto2.setCategory("Revenue");
                    dto2.setName("Revenue");
                    dto2.setAmount(l.getRevenue());
                    dtos.add(dto2);
                    AnalysisInternationalBUDataDto dto3 = new AnalysisInternationalBUDataDto();
                    dto3.setCategory("Profit");
                    dto3.setName("Profit");
                    dto3.setAmount(l.getProfit());
                    dtos.add(dto3);
                    AnalysisInternationalBUDataDto dto4 = new AnalysisInternationalBUDataDto();
                    dto4.setCategory("Trips");
                    dto4.setName("Trips");
                    dto4.setAmount(l.getTrips());
                    dtos.add(dto4);
                    AnalysisInternationalBUDataDto dto5 = new AnalysisInternationalBUDataDto();
                    dto5.setCategory("Subsidy");
                    dto5.setName("Subsidy B");
                    dto5.setAmount(l.getSubsidyB());
                    dtos.add(dto5);
                    AnalysisInternationalBUDataDto dto6 = new AnalysisInternationalBUDataDto();
                    dto6.setCategory("Subsidy");
                    dto6.setName("Subsidy C");
                    dto6.setAmount(l.getSubsidyC());
                    dtos.add(dto6);
                });*/
                objects.addAll(list);
                return objects;
            case 101:
                AnalysisInternationalTaxDataExample analysisInternationalTaxDataExample = new AnalysisInternationalTaxDataExample();
                analysisInternationalTaxDataExample.createCriteria().andPeriodEqualTo(period).andCompanyNameEqualTo(companyName).andCountryEqualTo(country);
                List<AnalysisInternationalTaxData> list2 = analysisInternationalTaxDataMapper.selectByExample(analysisInternationalTaxDataExample);
                List<AnalysisInternationalTaxDataDto> dtos2 = Lists.newArrayList();
                list2.forEach(l2->{
                    dtos2.add(beanUtil.copyProperties(l2,new AnalysisInternationalTaxDataDto()));
                });
                objects.addAll(dtos2);
                return objects;
            default:
                break;
        }
        return objects;
    }

    public HttpServletResponse downloadDomesticFile(HttpServletResponse response, AnalysisDomesticlParam param, String fileName) {
        String excelTemplatePathInClassPath = EnumAnalysisExpTempPath.getPath(param.getType());
        List<Object> datas = displayAnalysisImportData(param);
        if(datas.size()<1){
            throw new ServiceException(ErrorMessage.ExportFailed);
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(datas, excelTemplatePathInClassPath);
        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出国内税数据异常:%s",e.getMessage()));
        }
        return null;
    }

    public HttpServletResponse downloadInternationalFile(HttpServletResponse response, AnalysisInternationlParam param, String fileName) {
        String excelTemplatePathInClassPath = EnumAnalysisExpTempPath.getPath(param.getType());
        List<Object> datas = Lists.newArrayList();
        if(param.getType().equals(EnumAnalysisImportType.InternationalBuData.getCode())){
            Integer period = DateUtils.strToPeriod(param.getPeriod());
            String country = param.getCountry();
            String companyName = param.getCompanyName();
            AnalysisInternationalBusinessDataExample analysisInternationalBusinessDataExample = new AnalysisInternationalBusinessDataExample();
            analysisInternationalBusinessDataExample.createCriteria().andPeriodEqualTo(period).andCompanyNameEqualTo(companyName).andCountryEqualTo(country);
            List<AnalysisInternationalBusinessData> list = analysisInternationalBusinessDataMapper.selectByExample(analysisInternationalBusinessDataExample);
            datas.addAll(list);
        }else{
            datas = displayAnalysisInternationalImportData(param);
        }
        if(datas.size()<1){
            throw new ServiceException(ErrorMessage.ExportFailed);
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(datas, excelTemplatePathInClassPath);
        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出国际税数据异常:%s",e.getMessage()));
        }
        return null;
    }

    public List<String> getAnalysisInternationalCompanyList(Integer type, String periodStr) {
        List<String> companyList = Lists.newArrayList();
        Integer period = DateUtils.strToPeriod(periodStr);
        switch (type){
            case 100:
                companyList = analysisInternationalBusinessDataMapper.selectCompanyList(period);
                break;
            case 101:
                companyList = analysisInternationalTaxDataMapper.selectCompanyList(period);
                break;
            default:
                break;
        }
        return companyList;
    }

    public List<String> getAnalysisInternationalCountryList(Integer type, String periodStr) {
        List<String> countryList = Lists.newArrayList();
        Integer period = DateUtils.strToPeriod(periodStr);
        switch (type){
            case 100:
                countryList = analysisInternationalBusinessDataMapper.selectCountryList(period);
                break;
            case 101:
                countryList = analysisInternationalTaxDataMapper.selectCountryList(period);
                break;
            default:
                break;
        }
        return countryList;
    }

    private String getCellStringValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return cell.getStringCellValue();
        } else if(cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            // 取整
            return String.valueOf((int)cell.getNumericCellValue());
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Boolean getCellBolleanValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            switch (cell.getStringCellValue()) {
                case "TRUE":
                    return Boolean.TRUE;
                case "FALSE":
                    return Boolean.FALSE;
                case "是":
                    return Boolean.TRUE;
                case "否":
                    return Boolean.FALSE;
                default:
                    return null;
            }
        }else if (cell.getCellTypeEnum().equals(CellType.BOOLEAN)) {
            return cell.getBooleanCellValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Integer getCellIntegerValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return Integer.valueOf(cell.getStringCellValue());
        } else if(cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return (int)cell.getNumericCellValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private BigDecimal getCellBigDecimalValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return new BigDecimal(cell.getStringCellValue());
        } else if(cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return new BigDecimal(cell.getNumericCellValue());
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Long getCellLongDecimalValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return Long.valueOf(cell.getStringCellValue());
        } else if(cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return new Double(cell.getNumericCellValue()).longValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private boolean isSheetEmpty(Sheet sheet) {
        if (sheet.getLastRowNum() > 0 &&
                (null == sheet.getRow(0).getCell(0) || "Version".equals(sheet.getRow(0).getCell(0).getStringCellValue())) &&
                null == sheet.getRow(0).getCell(2)) {
            return true;
        }
        return false;
    }
}
