package pwc.taxtech.atms.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.util.DataUtil;
import pwc.taxtech.atms.common.util.FileUploadUtil;
import pwc.taxtech.atms.common.util.JxlsUtils;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.*;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dpo.CitAssetEamMappingDto;
import pwc.taxtech.atms.dpo.ReportDto;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.dto.export.ExportDto;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.exception.NotFoundException;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.dpo.DataSourceCellDataDto;
import pwc.taxtech.atms.vat.dpo.DataSourceExtendDto;
import pwc.taxtech.atms.vat.dpo.InputVATInvoiceItemExtendDto;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.impl.ReportGeneratorImpl;
import pwc.taxtech.atms.vat.service.impl.report.functions.FormulaHelper;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.locks.LockSupport;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.dto.vatdto.WrapPeriodJobDto.*;

/**
 * @author zhikai.z.wei
 */
@Component
public class CitReportServiceImpl extends BaseService {
    private final static Logger logger = LoggerFactory.getLogger(CitReportServiceImpl.class);
    private final static String[] functions = {"SGSR", "FSJZ", "ND", "BB", "XXFP", "GZSD", "PC", "JXFPMX",
            "JXFP", "PSUM", "DFFS", "JFFS", "WPSR", "WPNAME", "WPTYPE", "SUM2", "RSUMIF", "QMYE", "ZC"};

    @Autowired
    private ReportGeneratorImpl reportGenerator;
    @Resource
    private PeriodTaxRuleSettingMapper periodTaxRuleSettingMapper;
    @Resource
    private PeriodTaxPayerReportRuleMapper periodTaxPayerReportRuleMapper;
    @Resource
    private PeriodEnterpriseAccountMapper periodEnterpriseAccountMapper;
    @Autowired
    private InputVatInvoiceMapper inputVATInvoiceMapper;
    @Autowired
    private OutputVatInvoiceMapper outputVATInvoiceMapper;
    @Resource
    private CustomsInvoiceMapper customsInvoiceMapper;
    @Resource
    private PeriodCellDataSourceMapper periodCellDataSourceMapper;
    @Resource
    private KeyValueConfigMapper keyValueConfigMapper;
    @Resource
    private PeriodModifiedReportCellMapper periodModifiedReportCellMapper;
    @Resource
    private PeriodReportMapper periodReportMapper;
    @Resource
    private EnterpriseAccountMapper enterpriseAccountMapper;
    @Resource
    private CellTemplateConfigMapper cellTemplateConfigMapper;
    @Resource
    private CellTemplateMapper cellTemplateMapper;
    @Resource
    private PeriodDataSourceDetailMapper periodDataSourceDetailMapper;
    @Resource
    private PeriodDataSourceMapper periodDataSourceMapper;
    @Resource
    private PeriodCellDataMapper periodCellDataMapper;
    @Resource
    private PeriodFormulaBlockMapper periodFormulaBlockMapper;
    @Resource
    private PeriodCellTemplateConfigMapper periodCellTemplateConfigMapper;
    @Resource
    private PeriodCellTemplateMapper periodCellTemplateMapper;
    @Resource
    private PeriodTemplateMapper periodTemplateMapper;
    @Resource
    private TemplateMapper templateMapper;
    @Resource
    private ProjectServiceTypeMapper projectServiceTypeMapper;
    @Resource
    private TaxPayerReportRuleMapper taxPayerReportRuleMapper;
    @Resource
    private ProjectMapper projectMapper;
    @Resource
    private DistributedIdService distributedIdService;
    @Resource
    private PeriodJobMapper periodJobMapper;
    @Resource
    private PeriodApproveMapper periodApprovalMapper;
    @Resource
    RevenueConfigMapper revenueConfigMapper;
    @Autowired
    private HttpFileService httpFileService;
    @Autowired
    DidiFileUploadService didiFileUploadService;

    @Resource
    private TrialBalanceMapper trialBalanceMapper;
    @Resource
    private AdjustmentTableMapper adjustmentTableMapper;
    @Resource
    private ProfitLossStatementMapper profitLossStatementMapper;
    @Resource
    private JournalEntryMapper journalEntryMapper;
    @Autowired
    private DataUtil dataUtil;

    @Autowired
    private OrganizationServiceTemplateGroupMapper orgServiceTemplateGroupMapper;


    public OperationResultDto<List<ReportDto>> getReportTemplate(String projectId, EnumServiceType serviceType, Integer period) {
        OperationResultDto<List<ReportDto>> operationResult = new OperationResultDto<>();
        try {
            Project project = projectMapper.selectByPrimaryKey(projectId);
            String organizationId = project == null ? null : project.getOrganizationId();
            if (organizationId == null) {
                operationResult.setResult(false);
                operationResult.setResultMsg("NoOrganization");
                return operationResult;
            }
            Long templateGroupId;

            OrganizationServiceTemplateGroupExample oSTGExample = new OrganizationServiceTemplateGroupExample();
            OrganizationServiceTemplateGroupExample.Criteria oSTGCriteria = oSTGExample
                    .createCriteria();
            oSTGCriteria.andOrganizationIdEqualTo(organizationId);
            oSTGCriteria.andServiceTypeIdEqualTo(serviceType.getCode().toString());

            List<OrganizationServiceTemplateGroup> currentOrganizationServiceTemplateGroupList = orgServiceTemplateGroupMapper
                    .selectByExample(oSTGExample);
            Optional<Long> tempVlaue = orgServiceTemplateGroupMapper.selectByExample(oSTGExample).stream().map(OrganizationServiceTemplateGroup::getTemplateGroupId).findFirst();
            if (tempVlaue.isPresent()) {
                templateGroupId = tempVlaue.get();
            } else {
                templateGroupId = 0L;
            }

            Map<String, Object> map = new HashMap<>();
            map.put("period", 0);
            map.put("projectId", projectId);
            map.put("templateGroupID", templateGroupId);

            List<ReportDto> reportDtos = templateMapper.getTemplateLeftJoinReport(map);

            if (reportDtos.isEmpty()) {
                operationResult.setResultMsg("No Template");
                return operationResult;
            }

            operationResult.setResult(true);
            operationResult.setData(reportDtos);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResult.setResult(false);
        }
        return operationResult;
    }

    public OperationResultDto generateCitData(String projectId, EnumServiceType serviceType, Boolean isMergeManualData,
                                              Integer periodParam, Integer reportType, Optional<String> generator) {
        OperationResultDto operationResultDto = new OperationResultDto();
        //验证传入的业务类型参数和后台CIT所代表的业务类型是否一致
        MyAsserts.assertEq(serviceType, EnumServiceType.CIT, new NotFoundException());
        MyAsserts.assertNull(periodJobMapper.getRunningJob(projectId, periodParam), Exceptions.TASK_HAS_BEGINNING);
        String status = periodApprovalMapper.getStatusByProjectIdAndPeriod(projectId, periodParam);
        MyAsserts.assertTrue(status == null || !status.equals(Constant.APPROVAL_COMMITTED), Exceptions.REPORT_IN_PROCESS_OR_AGREED_EXCEPTION);
        try {
            //根据projectId获取相关报表
            List<Template> templates = getTemplatesByProjectId(projectId);

            //因WrapPeriodJobDto.createReportGenJob方法需要传入数组，但CIT并不需要所以传入一个空数组
            List<String> dataValidateArray = new ArrayList<String>();
            PeriodJob genJob = WrapPeriodJobDto.createReportGenJob(projectId, periodParam, templates, dataValidateArray);

            periodJobMapper.insert(genJob);
            new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        updateConfig(projectId, periodParam, isMergeManualData, templates, genJob);

                        // TODO 进行数据校验
//                        DataValidation(periodParam, projectId, genJob);

                        PeriodResources resources = reportGenerator.getPeriodResources(projectId, periodParam,
                                templates.stream().map(Template::getId).collect(Collectors.toList()));

                        Workbook workbook = reportGenerator.createWorkBookByPeriodTemplate(resources.getPeriodTemplates(), genJob);

                        reportGenerator.addCitFunctionsAndContext(workbook, functions, reportGenerator.initContext(resources, periodParam));
                        reportGenerator.setConfigAndDataToWorkBook(workbook, resources);

                        FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
                        evaluator.evaluateAll();

                        setStatus(genJob, STATUS_END);
                        periodJobMapper.updateByPrimaryKey(genJob);

                        reportGenerator.updateWorkbookCaclsValueToDb(projectId, periodParam, workbook, resources, isMergeManualData, genJob);
                        setStatus(genJob, STATUS_END);
                        periodJobMapper.updateByPrimaryKey(genJob);
                    } catch (Exception e) {
                        setStatus(genJob, STATUS_ERROR);
                        e.printStackTrace();
                        genJob.setErrorMsg("Sever error " + e.getClass().getName());
                        periodJobMapper.updateByPrimaryKey(genJob);
                    }
                }

            }).start();
            operationResultDto.setData(new PeriodJobDto().copyFromPeriodJob(genJob));
            operationResultDto.setResult(true);
        } catch (Exception ex) {
            operationResultDto.setResult(false);
            logger.error(ex.getMessage(), ex);
        }
        return operationResultDto;
    }

    /**
     * 根据projectId（卡片ID）获取模板
     *
     * @param projectId
     * @return
     */
    private List<Template> getTemplatesByProjectId(String projectId) {
        //验证参数projectId是否为空
        MyAsserts.assertNotEmpty(projectId, Exceptions.PROJECT_PROJECT_EXCEPTION);
        //验证根据参数projectId获取的project是否为null
        Project project = projectMapper.selectByPrimaryKey(projectId);
        MyAsserts.assertNotNull(project, Exceptions.NOT_FOUND_REPORT_EXCEPTION);
        //根据projectId和CIT所代表的数字类型获取报表模板ID
        Long templateGroupId = projectMapper.getTemplateGroupIdByProject(projectId, EnumServiceType.CIT.getCode());
        //验证查询得到的报表模板ID是否为null
        MyAsserts.assertNotNull(templateGroupId, Exceptions.NOT_FOUND_TEMPLATE_GROUP_EXCEPTION);
        //根据报表模板组ID查询在该组的报表模板
        List<Template> templates = queryTemplateByGroup(templateGroupId);
        MyAsserts.assertNotEmpty(templates, Exceptions.NOT_FOUND_TEMPLATE_EXCEPTION);

        templates.sort(Comparator.comparing(Template::getOrderIndex));
        return templates;
    }

    /**
     * 根据报表模板组ID查询在该组的报表模板
     *
     * @param templateGroupId
     * @return
     */
    private List<Template> queryTemplateByGroup(Long templateGroupId) {
        TemplateExample example = new TemplateExample();
        example.createCriteria().andTemplateGroupIdEqualTo(templateGroupId).andIsActiveAssociationEqualTo(true);
        return templateMapper.selectByExample(example);
    }

    /*  @Autowired
  private CitJour*/
    //数据校验
    private void DataValidation(Integer periodParam, String projectId, PeriodJob genJob) {
        setStatus(genJob, "DA004", STATUS_BEGIN);

        //本期余额表
      /*  TrialBalanceExample trialBalanceExample = new TrialBalanceExample();
        TrialBalanceExample.Criteria criteria = trialBalanceExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        criteria.andPeriodEqualTo(periodParam);
        TrialBalance trialBalances = trialBalanceMapper.selectByExample(trialBalanceExample).get(0);
        //本期利润表
        ProfitLossStatementExample profitLossStatementExample = new ProfitLossStatementExample();
        ProfitLossStatementExample.Criteria criteria2 = profitLossStatementExample.createCriteria();
        criteria2.andProjectIdEqualTo(projectId);
        criteria2.andPeriodEqualTo(periodParam);
        ProfitLossStatement profitLossStatements = profitLossStatementMapper.selectByExample(profitLossStatementExample).get(0);*/


        //调整表
        Map<String, Object> map1 = new HashedMap();
        map1.put("projectId", projectId);
        map1.put("period", periodParam);
        if ((periodParam + "").length() == 1) {
            map1.put("period", "0" + periodParam);
        } else {
            map1.put("period", "" + periodParam);
        }

        List<AdjustmentTable> adjustmentTables = adjustmentTableMapper.selectBeforeAdjustData(map1);
        Map<String, Object> map2 = new HashedMap();
        map2.put("projectId", projectId);
        if ((periodParam + "").length() == 1) {
            map2.put("period", "0" + periodParam);
        } else {
            map2.put("period", "" + periodParam);
        }
        Map<String, Object> map = new HashMap<>();
        map.put("period", periodParam);
        map.put("projectId", projectId);
        Map<String, Object> mapProject = dataUtil.getProjectById(projectId);
        map.put("companyCode", mapProject.get("code"));
        map.put("companyName", mapProject.get("name"));
        List<JournalEntry> journalEntries = journalEntryMapper.selectNowAdjustData(map2);
        for (int i = 0; i < adjustmentTables.size(); i++) {
            for (int j = 0; j < journalEntries.size(); j++) {
                JournalEntry journalEntry = journalEntries.get(j);
                AdjustmentTable adjustmentTable = adjustmentTables.get(j);
                if (journalEntry.getSegment3().equals(adjustmentTable.getSegment3()) &&
                        journalEntry.getSegment5().equals(adjustmentTable.getSegment5()) &&
                        journalEntry.getSegment6().equals(adjustmentTable.getSegment6()) && journalEntry.getPeriodJrMinDr() != adjustmentTable.getPeriodDrMixCr()) {
                    setStatus(genJob, STATUS_ERROR);
                    Constant.ReportDataValidateLog reportDataValidateLog = new Constant.ReportDataValidateLog(journalEntries.get(j).getSegment3(), journalEntries.get(j).getSegment5(), journalEntries.get(j).getSegment6());
                    //记录校验结果
                    //

                    map.put("result", reportDataValidateLog.getResult(false));
                    map.put("validateResult", "error");
                    map.put("tmsPeriod", journalEntry.getTmsPeriod());
                    map.put("organizationId", journalEntry.getOrganizationId());

                    insertDataValidateResult(map);
                    periodJobMapper.updateByPrimaryKey(genJob);
                    return;
                }
            }
        }
        setStatus(genJob, STATUS_END);
        map.put("validateResult", "success");
        map.put("result", "");
        map.put("tmsPeriod", 0);
        map.put("organizationId", mapProject.get("organization_id"));
        insertDataValidateResult(map);
        periodJobMapper.updateByPrimaryKey(genJob);
    }

    public void insertDataValidateResult(Map map) {
        DataValidateLog dataValidateLog = new DataValidateLog();
        dataValidateLog.setCreateTime(new Date());
        dataValidateLog.setPeriod((Integer) map.get("period"));
        dataValidateLog.setProjectId((String) map.get("projectId"));
        dataValidateLog.setValidateResult((String) map.get("validateResult"));
        dataValidateLog.setResult((String) map.get("result"));
        dataValidateLog.setOrganizationId((String) map.get("organizationId"));
        dataValidateLog.setCompanyCode((String) map.get("companyCode"));
        dataValidateLog.setCompanyName((String) map.get("companyName"));
        dataValidateLog.setTmsPeriod((Integer) map.get("tmsPeriod"));
        dataValidateLogMapper.insert(dataValidateLog);
    }

    @Resource
    private DataValidateLogMapper dataValidateLogMapper;

    /**
     * 清除配置，复制配置
     *
     * @param projectId
     * @param period
     * @param isMergeManualData
     * @param templates
     * @param job
     */
    private void updateConfig(String projectId, Integer period, Boolean isMergeManualData, List<Template> templates, PeriodJob job) {
        List<Long> exceptTemplateIds = templateMapper.getIdsForExceptTemplate();
        //清除配置
        clearPeriodData(projectId, period, exceptTemplateIds, isMergeManualData);
        //复制配置
        copyTemplateAndConfigFromAdmin(projectId, templates, period);

    }

    /**
     * 清除配置
     *
     * @param projectId
     * @param period
     * @param templateIds
     * @param isMergeManualData
     */
    private void clearPeriodData(String projectId, Integer period, List<Long> templateIds, boolean isMergeManualData) {
        if (templateIds.isEmpty()) templateIds.add(Long.MAX_VALUE);
        PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
        periodFormulaBlockExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodFormulaBlockMapper.deleteByExample(periodFormulaBlockExample);

        PeriodTaxRuleSettingExample periodTaxRuleSettingExample = new PeriodTaxRuleSettingExample();
        periodTaxRuleSettingExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodTaxRuleSettingMapper.deleteByExample(periodTaxRuleSettingExample);

        PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
        periodCellTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andReportTemplateIdNotIn(templateIds);
        periodCellTemplateMapper.deleteByExample(periodCellTemplateExample);

        PeriodCellTemplateConfigExample periodCellTemplateConfigExample = new PeriodCellTemplateConfigExample();
        periodCellTemplateConfigExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andReportTemplateIdNotIn(templateIds);
        periodCellTemplateConfigMapper.deleteByExample(periodCellTemplateConfigExample);

        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andTemplateIdNotIn(templateIds);
        periodTemplateMapper.deleteByExample(periodTemplateExample);

        PeriodTaxPayerReportRuleExample periodTaxPayerReportRuleExample = new PeriodTaxPayerReportRuleExample();
        periodTaxPayerReportRuleExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodTaxPayerReportRuleMapper.deleteByExample(periodTaxPayerReportRuleExample);

        PeriodDataSourceExample periodDataSourceExample = new PeriodDataSourceExample();
        periodDataSourceExample.createCriteria().andProjectIdEqualTo(projectId)
                .andPeriodEqualTo(period);
        List<PeriodDataSource> periodDataSources = periodDataSourceMapper.selectByExample(periodDataSourceExample);

        Set<Long> periodDsIds = Sets.newHashSet(Long.MAX_VALUE);
        List<Long> periodManualDsIds = new ArrayList<>();

        if (isMergeManualData) {
            periodDataSources.forEach(m -> {
                if (m.getType().intValue() != FormulaDataSourceType.KeyInSource.getCode().intValue()) {
                    periodDsIds.add(m.getId());
                } else {
                    periodManualDsIds.add(m.getId());
                }
            });

        } else {
            periodDsIds.addAll(periodDataSources.stream().map(m -> m.getId()).collect(Collectors.toList()));
        }

        PeriodDataSourceDetailExample periodDataSourceDetailExample = new PeriodDataSourceDetailExample();
        periodDataSourceDetailExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period)
                .andDataSourceIdIn(Lists.newArrayList(periodDsIds));
        periodDataSourceDetailMapper.deleteByExample(periodDataSourceDetailExample);

        PeriodCellDataSourceExample periodCellDataSourceExample = new PeriodCellDataSourceExample();
        periodCellDataSourceExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodCellDataSourceMapper.deleteByExample(periodCellDataSourceExample);


        PeriodDataSourceExample periodDataSourceDeleteExample = new PeriodDataSourceExample();
        periodDataSourceDeleteExample.createCriteria().andIdIn(Lists.newArrayList(periodDsIds));
        periodDataSourceMapper.deleteByExample(periodDataSourceDeleteExample);

        PeriodReportExample periodReportExample = new PeriodReportExample();
        periodReportExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodReportMapper.deleteByExample(periodReportExample);

        PeriodCellDataExample periodCellDataExample = new PeriodCellDataExample();
        periodCellDataExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodCellDataMapper.deleteByExample(periodCellDataExample);


    }

    /**
     * 复制各种配置
     *
     * @param projectId
     * @param templates
     * @param period
     */
    private void copyTemplateAndConfigFromAdmin(String projectId, List<Template> templates, Integer period) {
        List<PeriodTemplate> periodTemplateList = new ArrayList<>();
        //循环报表
        for (Template template : templates) {
            Long startTime = System.currentTimeMillis();
            logger.debug("template copy start: " + startTime);
            PeriodTemplate periodTemplate = new PeriodTemplate();
            CommonUtils.copyProperties(template, periodTemplate);
            periodTemplate.setId(distributedIdService.nextId());
            periodTemplate.setPeriod(period);
            periodTemplate.setName(template.getName());
            periodTemplate.setCode(template.getCode());
            periodTemplate.setPath(template.getPath());
            periodTemplate.setReportType(template.getReportType());
            periodTemplate.setTemplateGroupId(template.getTemplateGroupId());
            periodTemplate.setOrderIndex(template.getOrderIndex());
//            periodTemplate.setCreateTime(template.getCreateTime());
//            periodTemplate.setUpdateTime(template.getUpdateTime());
            periodTemplate.setIsSystemType(template.getIsSystemType());
            periodTemplate.setIsActiveAssociation(template.getIsActiveAssociation());
            periodTemplate.setParentId(StringUtils.isBlank(template.getParentId()) ? null : template.getParentId());
            periodTemplate.setTemplateId(template.getId());
            periodTemplate.setCreateBy(StringUtils.isBlank(template.getCreateBy()) ? "admin" : template.getCreateBy());
            periodTemplate.setUpdateBy(StringUtils.isBlank(template.getUpdateBy()) ? "admin" : template.getUpdateBy());
            periodTemplate.setProjectId(projectId);
            periodTemplateList.add(periodTemplate);

            //复制报表模板单元格定位及相关信息到卡片单元格模板
            copyPeriodCellTemplateFromCellTemplate(projectId, template.getId(), period);
            //复制单元格内容到卡片单元格内容
            copyPeriodConfigFromCellTemplateConfig(projectId, template.getId(), period);
        }

        periodTemplateMapper.batchInsert(periodTemplateList);
    }

    /**
     * 复制报表模板单元格定位及相关信息到卡片单元格模板
     *
     * @param projectId
     * @param templateId
     * @param period
     */
    private void copyPeriodCellTemplateFromCellTemplate(String projectId, Long templateId, Integer period) {
        CellTemplateExample cellTemplateExample = new CellTemplateExample();
        cellTemplateExample.createCriteria().andReportTemplateIdEqualTo(templateId);
        List<CellTemplate> cellTemplateList = cellTemplateMapper.selectByExample(cellTemplateExample);
        List<PeriodCellTemplate> periodCellTemplateList = new ArrayList<>();

        for (CellTemplate cellTemplate : cellTemplateList) {
            PeriodCellTemplate periodCellTemplate = new PeriodCellTemplate();
            CommonUtils.copyProperties(cellTemplate, periodCellTemplate);
            periodCellTemplate.setId(distributedIdService.nextId());
            periodCellTemplate.setPeriod(period);
            periodCellTemplate.setReportTemplateId(templateId);
            periodCellTemplate.setRowIndex(cellTemplate.getRowIndex());
            periodCellTemplate.setRowName(cellTemplate.getRowName());
            periodCellTemplate.setColumnIndex(cellTemplate.getColumnIndex());
            periodCellTemplate.setColumnName(cellTemplate.getColumnName());
            periodCellTemplate.setComment(cellTemplate.getComment());
//            periodCellTemplate.setCreateTime(cellTemplate.getCreateTime());
//            periodCellTemplate.setUpdateTime(cellTemplate.getUpdateTime());
            periodCellTemplate.setCellTemplateId(cellTemplate.getId());
            periodCellTemplate.setDataType(cellTemplate.getDataType());
            periodCellTemplate.setIsReadOnly(cellTemplate.getIsReadOnly() ? 1 : 0);
            periodCellTemplate.setCopyFromId(cellTemplate.getCopyFromId());
            periodCellTemplate.setCreateBy(StringUtils.isBlank(cellTemplate.getCreateBy()) ? "admin" : cellTemplate.getCreateBy());
            periodCellTemplate.setUpdateBy(StringUtils.isBlank(cellTemplate.getUpdateBy()) ? "admin" : cellTemplate.getUpdateBy());
            periodCellTemplate.setProjectId(projectId);
            periodCellTemplateList.add(periodCellTemplate);
        }

        CommonUtils.subListWithLen(periodCellTemplateList, CommonUtils.BATCH_NUM).forEach(batch -> {
            periodCellTemplateMapper.batchInsert(batch);

        });


    }

    /**
     * 复制单元格内容到卡片单元格内容
     *
     * @param projectId
     * @param templateId
     * @param period
     */
    private void copyPeriodConfigFromCellTemplateConfig(String projectId, Long templateId, Integer period) {
        //查询符合条件的单元格模板参数
        CellTemplateConfigExample cellTemplateConfigExample = new CellTemplateConfigExample();
        cellTemplateConfigExample.createCriteria().andReportTemplateIdEqualTo(templateId);
        List<CellTemplateConfig> cellTemplateConfigList = cellTemplateConfigMapper.selectByExample(cellTemplateConfigExample);
        if (cellTemplateConfigList.isEmpty()) {
            return;
        }

        //循环单元格模板参数，并赋值给卡片单元格参数
        List<PeriodCellTemplateConfig> periodCellTemplateConfigList = new ArrayList<>();
        for (CellTemplateConfig cellTemplateConfig : cellTemplateConfigList) {
            PeriodCellTemplateConfig periodCellTemplateConfig = new PeriodCellTemplateConfig();
            CommonUtils.copyProperties(cellTemplateConfig, periodCellTemplateConfig);

            periodCellTemplateConfig.setId(distributedIdService.nextId());
            periodCellTemplateConfig.setPeriod(period);
            periodCellTemplateConfig.setCellTemplateId(cellTemplateConfig.getCellTemplateId());
            periodCellTemplateConfig.setReportTemplateId(templateId);
            periodCellTemplateConfig.setDataSourceType(cellTemplateConfig.getDataSourceType());
            periodCellTemplateConfig.setFormula(StringUtils.isBlank(cellTemplateConfig.getFormula()) ? null : cellTemplateConfig.getFormula());
            periodCellTemplateConfig.setParsedFormula(null);
            periodCellTemplateConfig.setFormulaDescription(cellTemplateConfig.getFormulaDescription());
            periodCellTemplateConfig.setAccountCodes(cellTemplateConfig.getAccountCodes());
            periodCellTemplateConfig.setInvoiceType(cellTemplateConfig.getInvoiceType());
            periodCellTemplateConfig.setTaxRate(cellTemplateConfig.getTaxRate());
            periodCellTemplateConfig.setInvoiceAmountType(cellTemplateConfig.getInvoiceAmountType());
            periodCellTemplateConfig.setModelIds(cellTemplateConfig.getModelIds());
            periodCellTemplateConfig.setCreateBy(StringUtils.isBlank(cellTemplateConfig.getCreateBy()) ? "admin" : cellTemplateConfig.getCreateBy());
            periodCellTemplateConfig.setCreateTime(cellTemplateConfig.getCreateTime());
            periodCellTemplateConfig.setUpdateBy(StringUtils.isBlank(cellTemplateConfig.getUpdateBy()) ? "admin" : cellTemplateConfig.getUpdateBy());
            periodCellTemplateConfig.setUpdateTime(cellTemplateConfig.getUpdateTime());
            periodCellTemplateConfig.setInvoiceCategory(cellTemplateConfig.getInvoiceCategory());
            periodCellTemplateConfig.setFormulaDataSource(cellTemplateConfig.getFormulaDataSource());
            periodCellTemplateConfig.setValidation(cellTemplateConfig.getValidation());
            periodCellTemplateConfig.setParsedValidation(null);
            periodCellTemplateConfig.setValidationDescription(cellTemplateConfig.getValidationDescription());
            periodCellTemplateConfig.setVoucherKeyword(cellTemplateConfig.getVoucherKeyword());
            periodCellTemplateConfig.setCellTemplateConfigId(cellTemplateConfig.getId());
            periodCellTemplateConfig.setKeyValueParsedFormula(null);
            //赋值projectId（卡片ID），区分是哪个卡片里面的数据
            periodCellTemplateConfig.setProjectId(projectId);

            fixedParsedFormula(periodCellTemplateConfig);
            fixedAccountCode(periodCellTemplateConfig);

            periodCellTemplateConfigList.add(periodCellTemplateConfig);
        }

        CommonUtils.subListWithLen(periodCellTemplateConfigList, CommonUtils.BATCH_NUM).forEach(batch -> {
            periodCellTemplateConfigMapper.batchInsert(batch);
        });

    }

    /**
     * 固定转换公式
     *
     * @param pctc
     */
    private void fixedParsedFormula(PeriodCellTemplateConfig pctc) {
        if (pctc.getFormula() != null && pctc.getFormula().contains("@")) {
            String regex = "@[0-9A-Z.]+";
            Pattern pp = Pattern.compile(regex);
            Matcher mm = pp.matcher(pctc.getFormula());
            String replace_result = pctc.getFormula();
            Boolean replace = true;
            while (mm.find()) {
                KeyValueConfigExample keyValueConfigExample = new KeyValueConfigExample();
                keyValueConfigExample.createCriteria().andKeyCodeEqualTo(StringUtils.removeStart(mm.group(), "@"));
                Optional<KeyValueConfig> keyValueConfig = keyValueConfigMapper.selectByExample(keyValueConfigExample).stream().findFirst();
                if (keyValueConfig.isPresent() && StringUtils.isNotBlank(keyValueConfig.get().getFormula())) {
                    replace_result = replace_result.replace(mm.group(), keyValueConfig.get().getFormula());
                } else {
                    replace = false;
                }
            }

            if (replace) {
                pctc.setKeyValueParsedFormula(replace_result);
            }
            //todo: get the keyvalue from table add to keyvalue parsed formula
//                        KeyValueConfigExample skeyValueConfigExample = new KeyValueConfigExample();
//                        keyValueConfigExample.createCriteria().andKeyCodeEqualTo(StringUtils.removeStart(cellTemplateConfig.getFormula(), "@"));
//                        Optional<KeyValueConfig> keyValueConfig = keyValueConfigMapper.selectByExample(keyValueConfigExample).stream().findFirst();
//                        if (keyValueConfig.isPresent()) {
//                            periodCellTemplateConfig.setKeyValueParsedFormula(keyValueConfig.get().getFormula());
//                        }
        }
    }

    /**
     * 未知用途。。
     *
     * @param pctc
     */
    private void fixedAccountCode(PeriodCellTemplateConfig pctc) {

        if (StringUtils.isNotBlank(pctc.getAccountCodes())) {
            String[] acctCodes = pctc.getAccountCodes().split(",");
            EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
            enterpriseAccountExample.createCriteria().andStdCodeIn(Arrays.asList(acctCodes));

            List<EnterpriseAccount> enterpriseAccounts = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
            if (!enterpriseAccounts.isEmpty()) {
                List<String> acctCodeList = enterpriseAccounts.stream().map(EnterpriseAccount::getCode).collect(Collectors.toList());
                StringBuilder codes = new StringBuilder();
                for (String code : acctCodeList) {
                    codes.append(code).append(",");
                }
                pctc.setAccountCodes(StringUtils.removeEnd(codes.toString(), ","));
            }
        }
    }


    /**
     * 查询最新的任务
     *
     * @param projectId
     * @param period
     * @return
     */
    public PeriodJob getPeriodJob(String projectId, Integer period) {
        return periodJobMapper.getPeriodJob(projectId, period);
    }

    /**
     * 获取符合条件的PeriodJob以供前台判断job执行的阶段
     *
     * @param projectId
     * @param period
     * @param jobId
     * @return
     */
    public PeriodJob getJobStatus(String projectId, Integer period, String jobId) {
        PeriodJob job = periodJobMapper.getJobStatus(projectId, period, jobId);
        MyAsserts.assertNotNull(job, Exceptions.NOT_FOUND_EXCEPTION);
        return job;
    }

    public OperationResultDto<List<ReportDto>> getFilterReportTemplate(String projectId, EnumServiceType serviceType, Integer periodParam) {
        OperationResultDto<List<ReportDto>> result = new OperationResultDto<>();
        CommonUtils.copyProperties(getReportTemplate(projectId, serviceType, periodParam), result);
        if (!result.getResult() || CollectionUtils.isEmpty(result.getData())) {
            return result;
        }
        int period = periodParam != null ? periodParam : 0;
        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria().andProjectIdEqualTo(projectId)
                .andPeriodEqualTo(period);
        List<PeriodTemplate> periodTemplateList = periodTemplateMapper.selectByExample(periodTemplateExample);
        List<String> ids = Lists.newArrayList();
        periodTemplateList.stream().forEach(x -> ids.add(x.getTemplateId() + ""));

        List<ReportDto> reportDtos = result.getData().stream().filter(x -> {
            return x.getId() != null && ids.contains(x.getTemplateId());
        }).collect(Collectors.toList());
        result.setData(reportDtos);
        return result;
    }

    @Autowired
    private CitAssetsListMapper assetsListMapper;

    /**
     * 获取固定资产及EAM Mapping的数据
     */
    public PageInfo<CitAssetsListDto> getAssetEamMappingPage(CitAssetsListDto citAssetsListDto) throws Exception {
        logger.debug("获取固定资产及EAM Mapping的数据");
        Page page = PageHelper.startPage(citAssetsListDto.getPageInfo().getPageIndex(), citAssetsListDto.getPageInfo().getPageSize());
        CitAssetsList citAssetsList = new CitAssetsList();
        BeanUtils.copyProperties(citAssetsListDto, citAssetsList);
        List<CitAssetEamMappingDto> citAssetsLists = assetsListMapper.getAssetEamMapping(citAssetsList);
        List<CitAssetsListDto> citAssetsListDtos = new ArrayList<>();
        citAssetsListDtos = assetToAssetEamMapping(citAssetsListDtos, citAssetsLists);
//        for (CitAssetsList citAssetsListTemp:citAssetsLists){
//            CitAssetsListDto temp = new CitAssetsListDto();
//            BeanUtils.copyProperties(citAssetsListTemp,temp);
//
//            //计算处置损益  处置/赔偿收入-账面净值(年终剩余价值)
//            temp.setDisposalProfitAndLoss(temp.getCompensationSaleAmount().subtract(temp.getYearEndValue()));
//            //计算资产的税务净值 taxNetValue  原值-累计税务折旧/摊销
//            temp.setTaxNetValue(temp.getAcquisitionValue().subtract(temp.getTaxToCurrentYearDepreciationAmount()));
//            //处置的税收损益  处置/赔偿收入-资产的税务净值
//            temp.setDisposalTaxBenefit(temp.getCompensationSaleAmount().subtract(temp.getTaxNetValue()));
//            citAssetsListDtos.add(temp);
//        }

        PageInfo<CitAssetsListDto> pageInfo = new PageInfo<>(citAssetsListDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(citAssetsListDto.getPageInfo().getPageIndex());
        return pageInfo;
    }

    /**
     * 固资申报表导出(第二种方式)--正在使用
     *
     * @param citAssetsListDto
     * @param response
     * @return
     */
    public int exportAEMData2(CitAssetsListDto citAssetsListDto, HttpServletResponse response) {
        CitAssetsList citAssetsList = new CitAssetsList();
        BeanUtils.copyProperties(citAssetsListDto, citAssetsList);
        List<CitAssetEamMappingDto> citAssetsLists = assetsListMapper.getAssetEamMapping(citAssetsList);
        if (citAssetsLists.size() == 0) {
            return 0;
        }
        List<CitAssetsListDto> citAssetsListDtos = new ArrayList<>();
        assetToAssetEamMapping(citAssetsListDtos, citAssetsLists);
        ExportDto exportDto = new ExportDto();
        exportDto.setFileName("固资申报表");
        exportDto.setTemplateUrl(Constant.citTemplateUrl + "/assetEamMapping.xlsx");
        exportDto.setResponse(response);
        exportDto.setList(citAssetsListDtos);
        exportDto.setRelation(null);
        JxlsUtils.export(exportDto);
        return 1;
    }

    public List<CitAssetsListDto> assetToAssetEamMapping(List<CitAssetsListDto> citAssetsListDtos, List<CitAssetEamMappingDto> citAssetsLists) {
        for (CitAssetsList citAssetsListTemp : citAssetsLists) {
            CitAssetsListDto temp = new CitAssetsListDto();
            BeanUtils.copyProperties(citAssetsListTemp, temp);

            //计算处置损益  处置/赔偿收入-账面净值(年终剩余价值)
            temp.setDisposalProfitAndLoss(temp.getCompensationSaleAmount().subtract(temp.getYearEndValue()));
            //计算资产的税务净值 taxNetValue  原值-累计税务折旧/摊销
            temp.setTaxNetValue(temp.getAcquisitionValue().subtract(temp.getTaxToCurrentYearDepreciationAmount()));
            //处置的税收损益  处置/赔偿收入-资产的税务净值
            temp.setDisposalTaxBenefit(temp.getCompensationSaleAmount().subtract(temp.getTaxNetValue()));
            citAssetsListDtos.add(temp);
        }
        return citAssetsListDtos;
    }

    public OperationResultDto<ReportDataDto> getCellData(Long reportId, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        ReportDataDto dataDto = new ReportDataDto();
        try {
            PeriodReport report = periodReportMapper.selectByPrimaryKey(reportId);
            if (report == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoReport");
                return resultDto;
            }

            PeriodTemplateExample example = new PeriodTemplateExample();
            example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(report.getTemplateId()).andPeriodEqualTo(report.getPeriod());
            Optional<PeriodTemplate> reportTemplate = periodTemplateMapper.selectByExample(example).stream().findFirst();
            if (!reportTemplate.isPresent()) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoReportTemplate");
                return resultDto;
            }

            Project project = projectMapper.selectByPrimaryKey(report.getProjectId());
            if (project == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoProject");
                return resultDto;
            }

            PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
            periodTemplateExample.createCriteria()
                    .andProjectIdEqualTo(projectId)
                    .andTemplateGroupIdEqualTo(reportTemplate.get().getTemplateGroupId())
                    .andPeriodEqualTo(report.getPeriod())
                    .andIsActiveAssociationEqualTo(true);

            List<Long> templateIds = new ArrayList<>();
            templateIds.add(report.getTemplateId());
            List<CellCalcInfoDto> cellCfgList = reportGenerator.getCellCalcInfo(templateIds, report.getPeriod(), project.getId());

            PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
            cellDataExample.createCriteria().andPeriodEqualTo(report.getPeriod()).andProjectIdEqualTo(projectId).andReportIdEqualTo(reportId);

            List<PeriodCellData> currentCellDataList = periodCellDataMapper.selectByExample(cellDataExample); //这是取的是单元格中显示的值
            PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
            periodFormulaBlockExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(report.getPeriod())
                    .andReportIdEqualTo(reportId);
         /*   Map<String, Object> map = new HashMap<>();
            map.put("projectId", projectId);
            map.put("period", report.getPeriod());
            map.put("reportId", reportId);
            */
            List<PeriodFormulaBlock> formulaBlocks = periodFormulaBlockMapper.selectByExample(periodFormulaBlockExample);

            dataDto.setFormulaBlocks(formulaBlocks);
            //todo: 关键字数据源稍后再加
            // 获取通过KeyValueDataId关联的数据源，即公式数据源

            //todo: 模型数据源稍后再加
            // 获取通过ModelBackFillAmount关联的数据源，即模型数据源
            // TODO: Check模型回填数据源是否会有多对多的情况出现，如有需distinct

            //公式计算数据源 20180711 完成
            List<DataSourceExtendDto> manualDataSourceList = periodDataSourceMapper.getFormulaDataSource(reportId);
            //loop the datasource to add datasource detail information
            for (DataSourceExtendDto dseo : manualDataSourceList) {
                //只处理有明细数据的DataSource
                if (dseo.getType().equals(FormulaDataSourceType.InputInvoice.getCode())
                        || dseo.getType().equals(FormulaDataSourceType.OutputInvoice.getCode())
                        || dseo.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
                    //todo: add json datasource detail @ here. not only itemValue
                    List<String> itemValues = new ArrayList<>();
                    itemValues.add(getDataSourceDetailList(dseo.getId()).getData());
                    dseo.setItems(itemValues);
                } else if (dseo.getType().equals(FormulaDataSourceType.Report.getCode())) {
                    List<String> itemValues = new ArrayList<>();
                    //todo: get the datasource detail to add the items property
                    PeriodDataSourceDetailExample dataSourceDetailExample = new PeriodDataSourceDetailExample();
                    dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(dseo.getId());
                    List<PeriodDataSourceDetail> dataSourceDetailList = SpringContextUtil.periodDataSourceDetailMapper.selectByExample(dataSourceDetailExample);
                    dataSourceDetailList.forEach(a -> {
                        if (StringUtils.isNotBlank(a.getItemValue())) {
                            itemValues.add(a.getItemValue());
                        }
                    });
                    if (dataSourceDetailList.size() > 0 && dataSourceDetailList.get(0).getDataSourceType() != 6) {
                        dseo.setDataSourceType(convertType(dataSourceDetailList.get(0).getDataSourceType()));
                    }
                    dseo.setItems(itemValues);
                }
            }

            List<DataSourceDtoExtend> datasource = new ArrayList<>();
            manualDataSourceList.forEach(a -> {
                DataSourceDto dataSourceDto = new DataSourceDto();
                dataSourceDto.setId(a.getId().toString());
                dataSourceDto.setAmount(a.getAmount());
                dataSourceDto.setKeyinData(a.getKeyinData());
                dataSourceDto.setName(a.getName());
                dataSourceDto.setOperationType(a.getOperationType());
                dataSourceDto.setDescription(a.getDescription());
                dataSourceDto.setCreateTime(a.getCreateTime());
                if (a.getDataSourceType() != null && a.getDataSourceType() > 0) {
                    dataSourceDto.setType(convetToFType(a.getDataSourceType()));
                } else {
                    dataSourceDto.setType(a.getType());
                }
                dataSourceDto.setItems(a.getItems());
                dataSourceDto.setPeriod(a.getPeriod());
                dataSourceDto.setRowIndex(a.getRowIndex());
                dataSourceDto.setRowName(a.getRowName());
                dataSourceDto.setColumnIndex(a.getColumnIndex());
                dataSourceDto.setColumnName(a.getColumnName());
                dataSourceDto.setReportTemplateId(a.getReportTemplateId().toString());
                dataSourceDto.setRel_sql(a.getRelSql());

                if (a.getType().equals(FormulaDataSourceType.Report.getCode())) {
                    if (a.getItems().size() == 0) {
                        dataSourceDto.setDataSourceType(CellDataSourceType.Formula.getCode());
                    } else {
                        dataSourceDto.setDataSourceType(a.getDataSourceType());
                    }
                } else if (a.getType().equals(FormulaDataSourceType.KeyInSource.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.KeyIn.getCode());
                } else if (a.getType().equals(FormulaDataSourceType.InputInvoice.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.InputInvoice.getCode());
                } else if (a.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.Voucher.getCode());
                } else if (a.getType().equals(FormulaDataSourceType.OutputInvoice.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.OutputInvoice.getCode());
                } else {
                    dataSourceDto.setDataSourceType(0);
                }

                DataSourceDtoExtend dataSourceDtoExtend = new DataSourceDtoExtend();
                dataSourceDtoExtend.setItem1(a.getCellTemplateId().toString());
                dataSourceDtoExtend.setItem2(dataSourceDto);

                datasource.add(dataSourceDtoExtend);
            });
            dataDto.setManualDataSources(datasource);

            List<CellDataDto> cellDataDtoList = new ArrayList<>();
            cellCfgList.forEach(a -> {
                CellDataDto cellDataDto = new CellDataDto();
                cellDataDto.setColumnIndex(a.getColumnIndex());
                cellDataDto.setColumnName(a.getColumnName());
                cellDataDto.setRowIndex(a.getRowIndex());
                cellDataDto.setRowName(a.getRowName());
                cellDataDto.setCellTemplateId(a.getCellTemplateId().toString());
                cellDataDto.setReportTemplateId(a.getReportTemplateId().toString());
                cellDataDto.setDataType(a.getDataType());
                cellDataDto.setReportId(reportId.toString());
                cellDataDto.setIsReadOnly(a.getIsReadOnly());
                Optional<PeriodCellData> celldata = currentCellDataList.stream()
                        .filter(x -> x.getCellTemplateId().toString().equals(a.getCellTemplateId())).findFirst();
                celldata.ifPresent(cellData -> cellDataDto.setCellId(cellData.getId().toString()));
                cellDataDto.setFormula(a.getFormula());
                celldata.ifPresent(cellData -> cellDataDto.setCellValue(cellData.getData()));
                celldata.ifPresent(cellData -> cellDataDto.setKeyinData(cellData.getKeyinData()));
                celldata.ifPresent(cellData -> cellDataDto.setFormulaExp(cellData.getFormulaExp()));
                cellDataDto.setCellTemplateConfig(CellConfigTranslater.getConfigDto(a));
                List<DataSourceDtoExtend> entryList = datasource.stream()
                        .filter(y -> y.getItem1().equals(a.getCellTemplateId())).collect(Collectors.toList());
                List<DataSourceDto> dataSourceDtoList = new ArrayList<>();
                entryList.forEach(z -> {
                    if (z.getItem2().getItems() != null && !z.getItem2().getItems().isEmpty() && z.getItem2().getItems().get(0).contains("tag")) {
                        z.getItem2().getItems().forEach(m -> {
                            ReportCellDataSourceDto dto = JSON.parseObject(m, ReportCellDataSourceDto.class);
                            if (dto != null && dto.getReportTemplateId() != null && !Pattern.compile("\\D").matcher(dto.getReportTemplateId()).find()) {
                                PeriodTemplateExample periodTemplateExample1 = new PeriodTemplateExample();
                                periodTemplateExample1.createCriteria().andTemplateIdEqualTo(Long.parseLong(dto.getReportTemplateId()))
                                        .andPeriodEqualTo(report.getPeriod());
                                Optional<PeriodTemplate> optional = periodTemplateMapper.selectByExample(periodTemplateExample1).stream().findFirst();
                                if (optional.isPresent()) {
                                    dto.setReportName(optional.get().getName());
                                }
                            }

                            dataSourceDtoList.add(dto);
                        });
                    } else
                        dataSourceDtoList.add(z.getItem2());
                });
                cellDataDto.setDataSourceList(dataSourceDtoList);
                cellDataDtoList.add(cellDataDto);
            });

            PeriodModifiedReportCellExample modifiedReportCellExample = new PeriodModifiedReportCellExample();
            modifiedReportCellExample.createCriteria().andReportIdEqualTo(reportId);
            List<PeriodModifiedReportCell> modifiedReportCellList = periodModifiedReportCellMapper.selectByExample(modifiedReportCellExample);
            modifiedReportCellList.forEach(mrc -> {
                Optional<CellDataDto> cdd = cellDataDtoList.stream()
                        .filter(cellDataDto -> cellDataDto.getReportId().equals(mrc.getReportId())
                                && cellDataDto.getRowIndex() == mrc.getRow()
                                && cellDataDto.getColumnIndex() == mrc.getCol()).findFirst();
                if (cdd.isPresent()) {
                    if (!mrc.getOriginalValue().equals(mrc.getValue())) {
                        cdd.get().setModifiedReportCell(mrc);
                        cdd.get().setCellValue(mrc.getValue());
                        cdd.get().setIsModified(true);
                    }
                }
            });
            dataDto.setCellData(cellDataDtoList);
            resultDto.setData(dataDto);
            resultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
        }
        return resultDto;
    }

    private Integer convertType(Integer dataSourceType) {
        if (dataSourceType.equals(FormulaDataSourceDetailType.InputInvoiceDataSourceDto.getCode())) {
            return CellDataSourceType.InputInvoice.getCode();
        } else if (dataSourceType.equals(FormulaDataSourceDetailType.OutputInvoiceDataSourceDto.getCode())) {
            return CellDataSourceType.OutputInvoice.getCode();
        } else {
            return CellDataSourceType.Formula.getCode();
        }
    }


    public OperationResultDto<String> getDataSourceDetailList(Long datasSourceId) {
        OperationResultDto<String> operationResultDto = new OperationResultDto<>();
        PeriodDataSource dataSource = periodDataSourceMapper.selectByPrimaryKey(datasSourceId);
        String itemStrValue = StringUtils.EMPTY;

        if (dataSource.getType().equals(FormulaDataSourceType.InputInvoice.getCode())) {
            PeriodDataSourceDetailExample dataSourceDetailExample = new PeriodDataSourceDetailExample();
            dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(datasSourceId).andDataSourceTypeEqualTo(CellDataSourceType.InputInvoice.getCode());
            List<PeriodDataSourceDetail> dataSourceDetails = periodDataSourceDetailMapper.selectByExample(dataSourceDetailExample);
            List<InputVATInvoiceItemExtendDto> inputVATInvoiceItemExtendDtos = new ArrayList<>();
            for (PeriodDataSourceDetail dataSourceDetail : dataSourceDetails) {
                if (StringUtils.isNotBlank(dataSourceDetail.getItemValue())) {
                    InputVATInvoiceItemExtendDto inputVATInvoiceItemExtendDto = inputVATInvoiceMapper.selectInvoiceExtendDto(dataSourceDetail.getItemValue());
                    inputVATInvoiceItemExtendDtos.add(inputVATInvoiceItemExtendDto);
                }
            }
            itemStrValue = JSON.toJSONString(inputVATInvoiceItemExtendDtos);
        } else {
            itemStrValue = StringUtils.EMPTY;
        }

        operationResultDto.setData(itemStrValue);
        operationResultDto.setResult(true);
        return operationResultDto;
    }

    @Autowired
    private CitJournalEntryAdjustMapper citJournalEntryAdjustMapper;

    public OperationResultDto addCellManualDataSource(ManualDataSourceDto data, String projectId) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            if (data.getCellId() == null) {
                //todo: insert celldata for manual datasource
                PeriodCellData cellData = new PeriodCellData();
                cellData.setId(distributedIdService.nextId());
                cellData.setReportId(data.getReportId());
                cellData.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                cellData.setData(data.getAmount() == null ? null : data.getAmount().toString());
                cellData.setKeyinData(data.getKeyinData());
                cellData.setFormulaExp(data.getAmount() == null ? "0.0" : data.getAmount().toString());
                cellData.setCreateBy("admin");
                cellData.setCreateTime(new Date());
                cellData.setUpdateBy("admin");
                cellData.setUpdateTime(new Date());
                cellData.setProjectId(projectId);
                cellData.setPeriod(data.getPeriod());
                periodCellDataMapper.insertSelective(cellData);
                data.setCellId(cellData.getId());
            } else {
                PeriodCellData cellData = periodCellDataMapper.selectByPrimaryKey(data.getCellId());
                /*if (StringUtils.isNotBlank(data.getKeyinData())) {
                    cellData.setKeyinData(data.getKeyinData());
                    if (StringUtils.isEmpty(cellData.getFormulaExp()))
                        cellData.setFormulaExp(data.getKeyinData());
                    periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                }*/ /*else if (data.getAmount() != null && cellData.getData() != data.getAmount().toString()) {
                    cellData.setData(data.getAmount().toString());
                    if (StringUtils.isEmpty(cellData.getFormulaExp()))
                        cellData.setFormulaExp(data.getAmount().toString());
                    periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                }*//* else if (StringUtils.isNotBlank(data.getPenValue())) {//穿透调整数据
                    cellData.setData(data.getPenValue());
                    if (StringUtils.isEmpty(cellData.getFormulaExp()))
                        cellData.setFormulaExp(data.getAmount().toString());
                    periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                }*/
                boolean boo1 = StringUtils.isNotBlank(data.getPenValue());//穿透合并值
                boolean boo2 = StringUtils.isNotBlank(data.getKeyinData());//手工输入
                String sumValue = null;
                if (boo1 && boo2) {
                    sumValue = String.valueOf((Double.parseDouble(data.getPenValue()) + Double.parseDouble(data.getKeyinData())));

                } else if (boo1 && !boo2) {
                    sumValue = String.valueOf((Double.parseDouble(data.getPenValue())));
                } else if (!boo1 && boo2) {
                    sumValue = String.valueOf((Double.parseDouble(data.getKeyinData())));
                }
                if (StringUtils.isEmpty(cellData.getFormulaExp()))
                    cellData.setFormulaExp(data.getAmount().toString());
                cellData.setData(sumValue);
                periodCellDataMapper.updateByPrimaryKeySelective(cellData);


                /*kevin insert  */ // TODO: 3/21/2019  需要验证
                PeriodCellTemplateConfigExample example = new PeriodCellTemplateConfigExample();
                PeriodCellTemplateConfigExample.Criteria criteria = example.createCriteria();
                criteria.andCellTemplateIdEqualTo(Long.parseLong(data.getCellTemplateId()));
                criteria.andProjectIdEqualTo(data.getProjectId());
                PeriodCellTemplateConfig periodCellTemplateConfig = new PeriodCellTemplateConfig();
                periodCellTemplateConfig.setParsedFormula(sumValue);
                periodCellTemplateConfigMapper.updateByExampleSelective(periodCellTemplateConfig, example);
                //更改选中行相关数据
                CitJournalEntryAdjust citJournalEntryAdjust = new CitJournalEntryAdjust();
                citJournalEntryAdjust.setIsSelect("1");
                CitJournalEntryAdjustExample example1 = new CitJournalEntryAdjustExample();
                CitJournalEntryAdjustExample.Criteria criteria1 = example1.createCriteria();
                criteria1.andProjectIdEqualTo(data.getProjectId());
                criteria1.andSubjectCodeEqualTo(data.getAccountCode());
                citJournalEntryAdjustMapper.updateByExample(citJournalEntryAdjust, example1);

                JournalEntry journalEntry = new JournalEntry();
                journalEntry.setIsSelect("1");
                JournalEntryExample example2 = new JournalEntryExample();
                JournalEntryExample.Criteria criteria2 = example2.createCriteria();
                criteria2.andProjectIdEqualTo(data.getProjectId());
                criteria2.andSegment3EqualTo(data.getAccountCode());
                journalEntryMapper.updateByExample(journalEntry, example2);
            }

            List<DataSourceExtendDto> dataSourceExtendDtos = periodDataSourceMapper.getManualDataSource(data.getCellId());

            PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
            periodCellTemplateExample.createCriteria().andCellTemplateIdEqualTo(Long.parseLong(data.getCellTemplateId()))
                    .andPeriodEqualTo(data.getPeriod()).andProjectIdEqualTo(projectId);

            Optional<PeriodCellTemplate> periodCellTemplate = periodCellTemplateMapper.selectByExample(periodCellTemplateExample).stream().findFirst();

            BigDecimal originalAmount = new BigDecimal("0");
            PeriodDataSource dataSourceModel = null;
            if (dataSourceExtendDtos.size() > 0) {
                dataSourceModel = dataSourceExtendDtos.get(0).getDataSource();
                if (StringUtils.isBlank(data.getKeyinData()))
                    updateCellValueForDataSourceChange(dataSourceModel, data.getAmount());
                originalAmount = dataSourceModel.getAmount() != null ? dataSourceModel.getAmount() : new BigDecimal("0");
                dataSourceModel.setName(data.getName());
                dataSourceModel.setDescription(data.getDescription());
                dataSourceModel.setAmount(data.getAmount());
                dataSourceModel.setUpdateBy("Admin");
                dataSourceModel.setKeyinData(data.getKeyinData());
                dataSourceModel.setUpdateTime(new Date());
                periodDataSourceMapper.updateByPrimaryKeySelective(dataSourceModel);
            } else {
                dataSourceModel = new PeriodDataSource();
                Long cellDataSourceId = distributedIdService.nextId();
                dataSourceModel.setId(cellDataSourceId);
                dataSourceModel.setType(FormulaDataSourceType.KeyInSource.getCode());
                dataSourceModel.setName(data.getName());
                dataSourceModel.setDescription(data.getDescription());
                dataSourceModel.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                dataSourceModel.setAmount(data.getAmount());
                if (periodCellTemplate.isPresent()) {
                    dataSourceModel.setRowIndex(periodCellTemplate.get().getRowIndex());
                    dataSourceModel.setColumnIndex(periodCellTemplate.get().getColumnIndex());
                }
                dataSourceModel.setCreateTime(new Date());
                dataSourceModel.setUpdateTime(new Date());
                dataSourceModel.setCreateBy("admin");
                dataSourceModel.setKeyinData(data.getKeyinData());
                dataSourceModel.setUpdateBy("admin");
                dataSourceModel.setProjectId(projectId);
                dataSourceModel.setPeriod(data.getPeriod());

                PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                cellDataSource.setId(distributedIdService.nextId());
                cellDataSource.setCellDataId(data.getCellId());
                cellDataSource.setDataSourceId(cellDataSourceId);
                cellDataSource.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                cellDataSource.setCreateTime(new Date());
                cellDataSource.setUpdateTime(new Date());
                cellDataSource.setProjectId(projectId);
                cellDataSource.setPeriod(data.getPeriod());
                updateCellValueForDataSourceChange(cellDataSource.getCellDataId(), new BigDecimal("0"), data.getAmount());
                periodDataSourceMapper.insertSelective(dataSourceModel);
                periodCellDataSourceMapper.insertSelective(cellDataSource);
                dataSourceExtendDtos = periodDataSourceMapper.getManualDataSource(data.getCellId());
            }

            //todo: update the reference cell's data, such as if formula is ND(1)+A2,
            //todo: if A2 is changed manually, ND(1)+A2 result to changed, and datasource of A2 should be updated
            //1. use celltemplateid and period get template id, the change should be in the same sheet.if not in the same sheet, the report should be recalculate, can't changed in time.
            //2. use the celltemplateid get col and row number, get the cell name such as A2,B3 etc.
            String cellName = StringUtils.EMPTY;
            Long templateId = 0L;
            if (periodCellTemplate.isPresent()) {
                Integer colNum = periodCellTemplate.get().getColumnIndex();
                Integer rowNum = periodCellTemplate.get().getRowIndex() + 1;
                cellName = FormulaHelper.excelColIndexToStr(colNum + 1) + rowNum.toString();
                templateId = periodCellTemplate.get().getReportTemplateId();
            }
            //3. use the template id get the reportid, use the reportid get the celltemplateids, then use the celltemplateids and cell name get the datasource which should be updated.
            PeriodReportExample reportExample = new PeriodReportExample();
            reportExample.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(templateId).andPeriodEqualTo(data.getPeriod());
            Optional<PeriodReport> report = periodReportMapper.selectByExample(reportExample).stream().findFirst();
            if (report.isPresent()) {
                Long reportId = report.get().getId();
                PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
                cellDataExample.createCriteria().andReportIdEqualTo(reportId);
                List<PeriodCellData> cellDataList = periodCellDataMapper.selectByExample(cellDataExample);
                for (PeriodCellData cellData : cellDataList) {
                    if (cellData.getId() != data.getCellId() && cellData.getFormulaExp().contains(cellName)) {
                        PeriodCellDataSourceExample cellDataSourceExample = new PeriodCellDataSourceExample();
                        cellDataSourceExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(cellData.getPeriod()).andCellDataIdEqualTo(cellData.getId()).andCellTemplateIdEqualTo(cellData.getCellTemplateId());
                        List<PeriodCellDataSource> cellDataSourceList = periodCellDataSourceMapper.selectByExample(cellDataSourceExample);
                        for (PeriodCellDataSource cellDataSource : cellDataSourceList) {
                            Long dataSourceId = cellDataSource.getDataSourceId();
                            PeriodDataSource dataSource = periodDataSourceMapper.selectByPrimaryKey(dataSourceId);
                            if (dataSource.getDescription().equals(cellName) && StringUtils.isBlank(data.getKeyinData())) {
                                BigDecimal changeValue = data.getAmount().subtract(dataSource.getAmount());
                                dataSource.setAmount(data.getAmount());
                                dataSource.setUpdateBy("admin");
                                dataSource.setUpdateTime(new Date());
                                periodDataSourceMapper.updateByPrimaryKeySelective(dataSource);
                                if (!cellData.getData().equals("#VALUE!")) {
                                    if (StringUtils.isBlank(cellData.getData())) cellData.setData("0");
                                    cellData.setData(new BigDecimal(cellData.getData()).add(changeValue).toString());
                                } else {
                                    cellData.setData(new BigDecimal("0.0").add(changeValue).toString());
                                }
                                //cellData.setFormulaExp(cellData.getData());
                                cellData.setUpdateTime(new Date());
                                periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                            } else if (dataSource.getDescription().equals(cellName) && StringUtils.isNotBlank(data.getKeyinData())) {
                                dataSource.setKeyinData(data.getKeyinData());
                                dataSource.setUpdateBy("admin");
                                dataSource.setUpdateTime(new Date());
                                periodDataSourceMapper.updateByPrimaryKeySelective(dataSource);
                            }
                        }
                    }
                }
            }
            //4. before you do the setp3 ,you should get the changed value of datasource. then update the celldata use this change. just add the change value to celldata amount.
            //5. then save the datasource and save the celldata.
            data.setId(dataSourceExtendDtos.get(0).getId());
            operationResultDto.setResult(true);
            operationResultDto.setData(data);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    private void updateCellValueForDataSourceChange(PeriodDataSource dataSourceEntity, BigDecimal amount) {
        BigDecimal originalAmount = dataSourceEntity.getAmount();
        dataSourceEntity.setAmount(amount);
        BigDecimal aValue = dataSourceEntity.getAmount() != null ? dataSourceEntity.getAmount() : new BigDecimal("0");
        BigDecimal bValue = originalAmount != null ? originalAmount : new BigDecimal("0");

        BigDecimal amountChange = aValue.subtract(bValue);
        DataSourceCellDataDto dataSourceCellDataDto =
                periodDataSourceMapper.getManualDataSource2(dataSourceEntity.getId());
        BigDecimal cellVal = new BigDecimal("0");
        if (dataSourceCellDataDto != null && dataSourceCellDataDto.getCellData().getData() != null) {
            if (dataSourceCellDataDto.getCellDataSource().getOperationType().equals(EnumOperationType.Sub)) {
                amountChange = BigDecimal.ZERO.subtract(amountChange);
            }

            try {
                cellVal = new BigDecimal(dataSourceCellDataDto.getCellData().getData());
                if (StringUtils.isNotBlank(dataSourceCellDataDto.getCellData().getData())) {
                    Integer cellDataType = periodCellTemplateMapper
                            .getDataType(dataSourceCellDataDto.getCellData().getCellTemplateId()
                                    , dataSourceCellDataDto.getCellData().getReportId());
                    dataSourceCellDataDto.getCellData().setData(
                            FormulaHelper.roundValue(cellVal.add(amountChange),
                                    KeyValueConfigResultType.values()[cellDataType],
                                    false,
                                    null).toString());
                }
            } catch (NumberFormatException ex) {
                logger.error(ex.getMessage(), ex);
            }
        }
    }

    private void updateCellValueForDataSourceChange(Long cellDataId, BigDecimal originalAmount, BigDecimal newAmount) {
        BigDecimal aValue = newAmount != null ? newAmount : new BigDecimal("0");
        BigDecimal bValue = originalAmount != null ? originalAmount : new BigDecimal("0");

        BigDecimal amountChange = aValue.subtract(bValue);
        PeriodCellData cellData = periodCellDataMapper.selectByPrimaryKey(cellDataId);
        BigDecimal cellVal = new BigDecimal("0");
        try {
            if (cellData != null && StringUtils.isEmpty(cellData.getKeyinData())) {
                cellVal = new BigDecimal(cellData.getData());
                Integer cellDataType = periodCellTemplateMapper
                        .getDataType(cellData.getCellTemplateId()
                                , cellData.getReportId());
                cellData.setData(FormulaHelper.roundValue(cellVal.add(amountChange)
                        , KeyValueConfigResultType.values()[cellDataType], false, null).toString());
            }
        } catch (NumberFormatException ex) {
            logger.error(ex.getMessage(), ex);
        }
    }

    private Integer convetToFType(Integer cellDataSourceType) {
        if (cellDataSourceType.equals(CellDataSourceType.InputInvoice.getCode())) {
            return FormulaDataSourceType.InputInvoice.getCode();
        }
        if (cellDataSourceType.equals(CellDataSourceType.OutputInvoice.getCode())) {
            return FormulaDataSourceType.OutputInvoice.getCode();
        } else {
            return FormulaDataSourceType.Other.getCode();
        }
    }
}
