package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.MissingArgEval;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.vat.entity.*;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class JFFS extends FunctionBase implements FreeRefFunction {
    public JFFS(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 6) {
            return null;
        }

        String code = getStringParam(args[0], ec);//科目代码
        int year = getIntParam(args[1], ec);
        int period = getIntParam(args[2], ec);
        int sourceDataType = getIntParam(args[3], ec);//取值数据源
        String segment5 = null;
        if(!(args[4] instanceof MissingArgEval)){
            segment5 = getStringParam(args[4], ec);//利润中心代码
        }
        String segment6 = null;
        if(!(args[5] instanceof MissingArgEval)){
            segment6 = getStringParam(args[5], ec);//产品代码
        }

        String formulaExpression = "JFFS(\"" + code + "\"," + year + "," + period + ","
                + sourceDataType ;
        if(segment5!=null){
            formulaExpression+=",\""+segment5+"\"";
        }else{
            formulaExpression+=",";
        }
        if(segment6!=null){
            formulaExpression+=",\""+segment6+"\"";
        }else{
            formulaExpression+=",";
        }
        formulaExpression+=")";
        logger.debug(formulaExpression);

        year = getYear(year);

        Integer yearOffset = 0;
        period = FormulaHelper.getPeriod(period, yearOffset, formulaContext);
        String orgId = formulaContext.getOrganizationId();
        year = year + yearOffset;

        List<ReportCellDataSourceDto> dataSource = Lists.newArrayList();
        double result = 0.00;
        if (sourceDataType == 1) {
            result = countForTrialBalance(code, dataSource, period, year, orgId,segment5,segment6);
        } else if (sourceDataType == 2) {
            result = countForAdjBalance(code, dataSource, period, year, orgId,segment5,segment6);
        } else if (sourceDataType == 3) {
            result = countForTrialFinalBalance(code, dataSource, period, year, orgId,segment5,segment6);
        } else {
            return NumberEval.ZERO;
        }
        Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                FormulaDataSourceDetailType.FormulaDataSourceDto,
                new BigDecimal(result), period, formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
        saveFormulaBlock(period, ec, formulaExpression, new BigDecimal(result), dataSoureId, formulaContext.getProjectId());
        return new NumberEval(result);

    }

    private int pardePeriod(int period, int year) {
        return Integer.parseInt("" + year + (period > 9 ? period : ("0" + period)));
    }


    private double countForTrialBalance(String code, List<ReportCellDataSourceDto> contain, int period, int year, String orgId,String segment5,String segment6) {
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        TrialBalanceExample example = new TrialBalanceExample();
        TrialBalanceExample.Criteria c1 = example.createCriteria().andSegment3EqualTo(code).andPeriodEqualTo(pardePeriod(period, year));
        if (organization != null) {
            c1.andOrganizationIdEqualTo(organization.getId());
        }
        if(StringUtils.isNotBlank(segment5)){
            c1.andSegment5EqualTo(segment5);
        }
        if(StringUtils.isNotBlank(segment6)){
            c1.andSegment5EqualTo(segment6);
        }
        List<TrialBalance> list = SpringContextUtil.trialBalanceMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(list)) {
            return 0.0;
        }
        List<TrialBalance> temp = list.stream().filter(x -> code.equalsIgnoreCase(x.getSegment3())).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(temp)) {
            temp = list;
        }

        for (TrialBalance balance : temp) {
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(balance.getPeriodDr());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return temp.stream().mapToDouble(a -> a.getPeriodDr().doubleValue()).sum();
    }

    private double countForAdjBalance(String code, List<ReportCellDataSourceDto> contain, int period, int year, String orgId,String segment5,String segment6) {
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        AdjustmentTableExample example = new AdjustmentTableExample();
        AdjustmentTableExample.Criteria c1 = example.createCriteria().andSegment3EqualTo(code).andPeriodEqualTo(pardePeriod(period, year));
        if (organization != null) {
            c1.andOrganizationIdEqualTo(organization.getId());
        }
        if(StringUtils.isNotBlank(segment5)){
            c1.andSegment5EqualTo(segment5);
        }
        if(StringUtils.isNotBlank(segment6)){
            c1.andSegment5EqualTo(segment6);
        }
        List<AdjustmentTable> list = SpringContextUtil.adjustmentTableMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(list)) {
            return 0.0;
        }
        List<AdjustmentTable> temp = list.stream().filter(x -> code.equalsIgnoreCase(x.getSegment3())).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(temp)) {
            temp = list;
        }

        for (AdjustmentTable balance : temp) {
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(balance.getPeriodDrBeq());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return temp.stream().mapToDouble(a -> a.getPeriodDrBeq().doubleValue()).sum();
    }

    private double countForTrialFinalBalance(String code, List<ReportCellDataSourceDto> contain, int period, int year, String orgId,String segment5,String segment6) {
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        TrialBalanceFinalExample example = new TrialBalanceFinalExample();
        TrialBalanceFinalExample.Criteria c1 = example.createCriteria().andSegment3EqualTo(code).andPeriodEqualTo(pardePeriod(period, year));
        if (organization != null) {
            c1.andOrganizationIdEqualTo(organization.getId());
        }
        if(StringUtils.isNotBlank(segment5)){
            c1.andSegment5EqualTo(segment5);
        }
        if(StringUtils.isNotBlank(segment6)){
            c1.andSegment5EqualTo(segment6);
        }
        List<TrialBalanceFinal> list = SpringContextUtil.trialBalanceFinalMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(list)) {
            return 0.0;
        }
        List<TrialBalanceFinal> temp = list.stream().filter(x -> code.equalsIgnoreCase(x.getSegment3())).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(temp)) {
            temp = list;
        }

        for (TrialBalanceFinal balance : temp) {
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(balance.getPeriodDr());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.Report.getCode());
            contain.add(dto);
        }
        return temp.stream().mapToDouble(a -> a.getPeriodDr().doubleValue()).sum();
    }
}
